<!DOCTYPE html>
<html lang="en">
  <head>
    {% include head.html %}
  </head>
  <body>
    <div class="content">
      {{ content }}
    </div>
    <footer>
      <span style="float:right;">
        Released under the <a href="http://opensource.org/licenses/BSD-3-Clause">BSD License</a>.
      </span>
      Copyright 2012-2019 <a href="http://hamcrest.org">hamcrest.org</a>
    </footer>
    <a href="http://github.com/hamcrest/JavaHamcrest"><img style="position: absolute; top: 0; right: 0; border: 0;" src="https://s3.amazonaws.com/github/ribbons/forkme_right_green_007200.png" alt="Fork me on GitHub"></a>
  </body>
</html>
