<head>
  <meta charset="UTF-8">
  <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
  <meta name="description" content="Hamcrest"/>
  <meta name="keywords" content="hamcrest java matchers junit testing"/>
  <meta name="robots" content="index, follow"/>
  <meta name="date" content="2018-11-01"/>
  <title>{{ page.title }}</title>
  <link href="css/hamcrest.css" rel="stylesheet" type="text/css">
  <link href="css/pygments-default.css" rel="stylesheet" type="text/css">
</head>
